/* axmodemscan.c
 *
 * Copyright (C) 2001 RedHat, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */
/*
 * Modified for SIKIGAMI 2001,2002 Axe,Inc.
 */

#include "axmodemscan.h"
#include "axmodem.h"
#include <time.h>
#include <assert.h>
#include <dirent.h>
#include <ctype.h>
#include <sys/stat.h>
#include <string.h>
#include <malloc.h>
#include <stdlib.h>
#include <stdio.h>
#include <syslog.h>
#include <glib.h>
#include "misc.h"


static char *commands[AxModemScan_NUM_STAGES] = {
    NULL,
		"Q0 V1 E1",
		"Z",
		"S0=0",
    "&C1",
		"&D2",
		"S11=55",
		"+FCLASS=0",
		NULL,
    NULL,
		"",
		NULL
};


void
AxModemScan_init_valued( AxModemScan * modem, char *devname)
{
    modem->stage = AxModemScan_Startup;
    memset(modem->status, 0, sizeof(modem->status));

    if (devname[0] == '/') {
			modem->file = (char *)g_malloc(sizeof(char)*(strlen(devname)+1));
			strcpy( modem->file, devname );
		}
    else {
			modem->file = (char *)g_malloc(sizeof(char)*(strlen(devname)+6));
			sprintf( modem->file, "/dev/%s", devname );
		}
    
    modem->baud = 1200;
    modem->modem = NULL;
    modem->tries = 0;
    modem->broken = 0;
}

void
AxModemScan_clear(AxModemScan*modem)
{
//    if (   AxModemScan_isok(modem)
//				&& AxMoedmScan_isdone(modem) )
    
  if (modem->modem) {
		AxModem_clear( modem->modem );
		g_free( modem->modem );
	}	
}

int
AxModemScan_isok( AxModemScan*modem )
{
    return ! modem->broken;
}

int
AxModemScan_isdone( AxModemScan*modem )
{
	return modem->stage == AxModemScan_Done;
}

int
AxModemScan_get_stage( AxModemScan*modem )
{
	return modem->stage;
}

char *
AxModemScan_initstr( AxModemScan*modem )
{
    char	s[200], * trim, * ret;
		int		i;

    strcpy(s, "AT");
    
    for( i=0; i<AxModemScan_NUM_STAGES; i++) {
			if (   modem->status[i] != AxModemScan_Worked
				  && modem->status[i] != AxModemScan_Test )
	 		  continue;
			if (!commands[i] || !commands[i][0])
		    continue;
			if (   (commands[i][0]=='Z' || commands[i][0]=='I')
					&& modem->status[i] != AxModemScan_Test)
		    continue;
	
			strcat(s, commands[i]);
			strcat(s, " ");
    }
    
		trim = trimspace(s);
		ret = (char*)g_malloc(sizeof(char)*(strlen(trim)+1));
		strcpy( ret, trim );

		return ret;
}

void
AxModemScan_execute( AxModemScan * modem)
{
 char * wk[2];

    if (	  AxModemScan_isdone(modem)
				|| !AxModemScan_isok(  modem)	) {
			return;
		}

    switch( modem->stage ) {
    	case AxModemScan_Startup:
//				assert(!modem->modem);
				modem->modem = (AxModem*)g_malloc(sizeof(AxModem));
				AxModem_init_valued(modem->modem,modem->file, modem->baud);
				modem->modem->die_fast = 0;
				if (!AxModem_isok(modem->modem)) {
			    if (AxStream_geterr((AxStream*)modem->modem)
							&& AxStream_geterr((AxStream*)modem->modem) != EIO
							&& AxStream_geterr((AxStream*)modem->modem) != ENOENT
							&& AxStream_geterr((AxStream*)modem->modem) != ENODEV) {
//						debug(WvLog::Info, "%s\n", modem->errstr());
			    }
	 		    modem->broken = 1;
				}
				else
			    modem->stage++;
				break;
	
	    case AxModemScan_AT:
 		  case AxModemScan_ATZ:
    	case AxModemScan_ATS0:
    	case AxModemScan_Carrier:
    	case AxModemScan_DTR:
    	case AxModemScan_FastDial:
    	case AxModemScan_FCLASS:
    	case AxModemScan_Reinit:
				assert(modem->modem);
				modem->status[modem->stage] = AxModemScan_Test;
				wk[1] = AxModemScan_initstr(modem);
				wk[0] = (char*)g_malloc(sizeof(char)*(strlen(wk[1])+2));
				sprintf( wk[0], "%s\r", wk[1] );
				if (   ! AxModemScan_doresult(modem,wk[0],modem->stage==AxModemScan_ATZ?3000:500)
	    			|| (   (modem->stage<=AxModemScan_ATZ||modem->stage==AxModemScan_Reinit)
						     && modem->status[modem->stage]==AxModemScan_Fail ) ) {
	    		modem->tries++;
	    
	    		if (modem->tries >= 3) {
//						debug("nothing.\n");
						modem->broken = 1;
	    		}
				}
				else {
			    modem->tries = 0;
	 			  modem->stage++;
				}
				g_free( wk[0] );
				g_free( wk[1] );
				break;
	
    case AxModemScan_GetIdent:
			assert(modem->modem);
			modem->status[modem->stage] = AxModemScan_Test;
//			debug("Modem Identifier: ");
			if (   ! AxModemScan_doresult( modem, "ATI\r", 500)
					|| (modem->status[modem->stage]==AxModemScan_Fail) ) {
		    modem->tries++;

//		    if (modem->tries >= 3)
//		    	debug("nothing.\n");

			}
			else {
//		    if (AxModemScan_is_isdn(modem))
//		    	debug("Looks like an ISDN modem.\n");
		    modem->tries = 0;
		    modem->stage++;
			}
	
    case AxModemScan_BaudStep:
			assert(modem->modem);
			AxStream_drain( (AxStream*)modem->modem );
			AxModem_speed(modem->modem,modem->baud*2);

			if (modem->tries>=3||AxModemBase_getspeed((AxModemBase*)modem->modem)<=modem->baud) {
		    modem->baud = AxModem_speed(modem->modem,modem->baud);
//		    debug("Max speed is %s; ", baud);
		    if (AxModemScan_is_isdn(modem)) {
					AxModem_speed(modem->modem,115200);
	   		 	modem->baud = AxModemBase_getspeed((AxModemBase*)modem->modem);
//		    	debug("using %s for ISDN modem.\n", baud);
		    }
		    else {
					if ( AxModemBase_getspeed((AxModemBase*)modem->modem) < 115200) {
				    modem->baud = AxModem_speed(modem->modem,modem->baud - 1);
//				    debug("using %s to be safe.\n", baud);
					}
					else {
//				    debug("that should be safe.\n");
					}
		    }
	    
		    modem->stage++;
		    modem->status[modem->stage] = AxModemScan_Worked;
		    break;
			}
	
//			debug("Speed %s: ", modem->getspeed());
	
			if (   !AxModemScan_doresult(modem,"AT\r", 500)
					|| modem->status[modem->stage] == AxModemScan_Fail) {
		    modem->tries++;
			}
			else {
		    modem->baud = AxModemBase_getspeed((AxModemBase*)modem->modem);
		    modem->tries = 0;
			}
			break;
	
    case AxModemScan_Done:
    case AxModemScan_NUM_STAGES:
			assert(0);
			break; // should never happen
  }

    
  if (modem->stage == AxModemScan_Done) {
		if (modem->modem) {
			AxModem_clear( modem->modem );
	    g_free( modem->modem );
		}
		modem->modem = NULL;
  }
}
char *
AxModemScan_filename( AxModemScan * modem )
{
	return modem->file;
}
int	
AxModemScan_maxbaud( AxModemScan * modem )
{
	return modem->baud;
}
int
AxModemScan_doresult(AxModemScan*modem,char*_s, int msec)
{
    char buf[1024], *cptr;
    size_t len;
    char * s;

    s = (char*)g_malloc(sizeof(char)*(strlen(_s)+1));
		strcpy( s, _s );

    AxStream_drain( (AxStream*)modem->modem );
    usleep(50 * 1000); // delay a bit after emptying the buffer
    AxStream_write( (AxStream*)modem->modem, s, strlen(s));

		g_free( s );

//    debug("%s -- ", trim_string(s.edit()));
    
    len = AxModemScan_coagulate( modem, buf, sizeof(buf), msec);

    if (!len) {
			return 0;
    }
    
    buf[len] = 0;
    
    cptr = trimspace(buf);

    while (strchr(cptr, '\r')) {
			cptr = trimspace(strchr(cptr, '\r')+1);
			if (	 modem->stage == AxModemScan_GetIdent
					&& modem->status[modem->stage] == AxModemScan_Test) {
	    	char *p = strpbrk(cptr, "\n\r");
	    	if (p) *p=0;
				modem->identifier = (char*)g_malloc(sizeof(char)*(strlen(cptr)+1));
				strcpy( modem->identifier, cptr );
	    	modem->status[modem->stage] = AxModemScan_Worked;
//	    	debug("%s\n", identifier);
	 	  	return 0;
			}
    }

    while (strchr(cptr, '\n')) {
			cptr = trimspace(strchr(cptr, '\n')+1);
    }

//    debug("%s\n", cptr);

    if (!strncmp(cptr, "OK", 2))
			modem->status[modem->stage] = AxModemScan_Worked;
    else
			modem->status[modem->stage] = AxModemScan_Fail;
    
    return 0;
}

size_t
AxModemScan_coagulate(AxModemScan*modem,char*buf,size_t size,int msec)
{
    size_t len, amt;
    char *cptr = buf;

		len = 0;

    assert(modem->modem);
    
    if (!AxModem_isok(modem->modem)) {
			modem->broken = 1;
			return 0;
    }
    
    while (AxStream_select((AxStream*)modem->modem,msec,1,0,0)) {
			amt = AxStream_read((AxStream*)modem->modem,cptr,size-1);
			cptr[amt] = 0;

			len += amt;
			size -= amt;
			cptr += amt;

			if (strstr(buf,"OK") || strstr(buf,"ERROR"))
	    	break;
    }
    
    return len;
}

char *
AxModemScan_is_isdn( AxModemScan * modem )
{
    if (!modem->identifier)
    	return NULL;

    if (modem->identifier == "3C882")		// 3Com Impact IQ
    	return modem->identifier;
    if (modem->identifier == "346800")		// USR ISDN TA
    	return modem->identifier;
    if (modem->identifier == "960")		// Motorola BitSurfr
    	return modem->identifier;

    return NULL;
}	

int
fileselect(const struct dirent *e)
{
    return !strncmp(e->d_name, "ttyS", 4);
       // (no internal ISDN support)   || !strncmp(e->d_name, "ttyI", 4);
}


int
filesort(const void *_e1, const void *_e2)
{
    struct dirent * *e1 = (struct dirent **)_e1;
    struct dirent * *e2 = (struct dirent **)_e2;
    char *p1, *p2;
    int diff;
    
    for (p1=(*e1)->d_name, p2=(*e2)->d_name; *p1 || *p2; p1++, p2++) {
			if (!isdigit(*p1) || !isdigit(*p2)) {
	    	diff = *p1 - *p2;
	    	if (diff) return diff;
			}
			else {
	    	return atoi(p1) - atoi(p2);
			}
    }
    
    return 0;
}

void
AxModemScanList_init( AxModemScanList * list )
{
	list->list = NULL;

	AxModemScanList_setup( list );
}
void
AxModemScanList_clear( AxModemScanList * list )
{
	GSList * ptr;

	AxModemScanList_shutdown( list );

	for( ptr=list->list; ptr; ptr=g_slist_next(ptr) ) {
		if ( ptr->data ) {
			AxModemScan_clear( (AxModemScan*)ptr->data );
			g_free( ptr->data );
			ptr->data = NULL;
		}
	}

	g_slist_free( list->list );
	list->list = NULL;
}
void
AxModemScanList_setup( AxModemScanList * list )
{
    struct dirent **namelist;
    struct stat mouse, modem;
    int num, count, mousestat, modemstat;
		AxModemScan * modemScan;
    
    list->thisline = -1;
    list->printed = 0;
    
    mousestat = stat("/dev/mouse", &mouse);
    modemstat = stat("/dev/modem", &modem);
    num = scandir("/dev", &namelist, fileselect, filesort);
    
    if (num < 0)
			return;
    
    for (count = 0; count < num; count++) {
			if (mousestat==0 && mouse.st_ino == (ino_t)namelist[count]->d_ino) {
//		    syslog(LOG_ERR,"\nIgnoring %s because /dev/mouse is a link to it.\n",
//								       namelist[count]->d_name);
		    continue;
			}
	
			modemScan = (AxModemScan*)g_malloc(sizeof(AxModemScan) );
			AxModemScan_init_valued( modemScan, namelist[count]->d_name );

			if (modemstat==0 && modem.st_ino == (ino_t)namelist[count]->d_ino)
	 	  	list->list = g_slist_prepend( list->list, modemScan );
			else
	 	  	list->list = g_slist_append( list->list, modemScan );
    }
    
    while (--num >= 0)
			g_free(namelist[num]);

    g_free(namelist);
}
void
AxModemScanList_shutdown( AxModemScanList * list )
{
//    delete log;
}

void
AxModemScanList_execute( AxModemScanList * list )
{
		GSList * ptr;
		AxModemScan * modem;

    assert (!AxModemScanList_isdone(list));

		for( ptr=list->list; ptr; ptr=g_slist_next(ptr) ) {
			if (!AxModemScan_isdone((AxModemScan*)ptr->data)) {
				break;
			}
		}

    if (!ptr) return; 

		modem = (AxModemScan*)ptr->data;
    
    if (!AxModemScan_isok(modem)) {
			if (!list->printed) {
		    char * f = AxModemScan_filename( modem );
		    char *cptr = strrchr(f, '/');
		    if (cptr)
					cptr++;
		    else
					cptr = f;

		    if (!strncmp(cptr, "tty", 3))
					cptr += 3;
	    
		    ++list->thisline;
				list->thisline = list->thisline % 8;
//		    if (!list->thisline)
//					syslog(LOG_INFO,"\n");

//		    syslog(LOG_INFO,"%-4s ",cptr);
			}

			list->list = g_slist_remove( list->list, (gpointer)modem );
			AxModemScan_clear( modem );
			list->printed = 0;
    }
    else {
			AxModemScan_execute( modem );
	
			if (AxModemScan_isok(modem))
		    list->printed = 1;
    }
	    
//    if (AxModemScanList_isdone(list))
//			syslog(LOG_INFO,"\n");
}

int
AxModemScanList_isdone(AxModemScanList*list)
{
 GSList * ptr;

    for( ptr=list->list; ptr; ptr=g_slist_next(ptr) )
			if (!AxModemScan_isdone((AxModemScan*)ptr->data))
				return 0;
    
    return 1;
}
