#include <config.h>
#include "modem.h"
#include "axconf.h"
#include <ctype.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include <stdio.h>
#include "mainwin.h"
#include "axmodemscan.h"
#include "misc.h"

static gint pid = 0;
static guint msg_input_handler;
static guint format_input_handler;
static gint end_count;

static gboolean
check_conf_section (gchar *section, AxConf *cfg)
{
	const gchar *test;
	gint test_int;

	/* other fields are optional. These are mandatory */
	test = AxConf_get(cfg,section, "Modem");
	if (test == NULL)
		return FALSE;
	test_int = AxConf_getint(cfg,section,"Baud",0);
	if (test_int == 0)
		return FALSE;

	test = AxConf_get(cfg,section, "Init1");
	if (test == NULL)
		return FALSE;

	return TRUE;
}

ConfStatus
check_conf ( AxConf *cfg)
{
	ConfStatus retval;
	int free_conf = FALSE;

	if (!g_file_test (get_config_path(), G_FILE_TEST_ISFILE))
		return CONF_NONEXISTENT;

	if (cfg == NULL) {
		cfg = (AxConf*)g_malloc( sizeof(AxConf) );
		AxConf_init_valued( cfg, get_config_path() );
		free_conf = TRUE;
	}

	if (!AxConf_isok(cfg))
		return CONF_NONEXISTENT;

	if (!check_conf_section ("Modem0", cfg))
		return CONF_NONEXISTENT;

	retval = CONF_MODEM_FOUND;

	if (check_conf_section ("Modem1", cfg))
		retval = CONF_MODEMS_FOUND;

	if (check_conf_section ("Dialer Defaults", cfg)) {
		if (retval == CONF_MODEM_FOUND)
			retval = CONF_MODEM_DEFAULT_FOUND;
		else
			retval = CONF_MODEMS_DEFAULT_FOUND;
	}
	if (free_conf) {
		AxConf_clear( cfg );
		g_free( cfg );
	}

	return retval;
}

void
copy_sections (AxConf *cfg, gchar *section1, gchar *section2)
{
	char *val;
	gint baud;

	g_return_if_fail(cfg != NULL);
	g_return_if_fail(section1 != NULL);
	g_return_if_fail(section2 != NULL);

	val = AxConf_get( cfg, section1, "Modem");
	AxConf_set(cfg,section2, "Modem", val);

	baud = AxConf_getint(cfg, section1, "Baud", 0);
	AxConf_setint(cfg, section2, "Baud", baud);

	val = AxConf_get( cfg, section1, "Init1");
	AxConf_set( cfg, section2, "Init1", val);

	val = AxConf_get( cfg, section1, "Init2");
	AxConf_set( cfg, section2, "Init2", val);

	val = AxConf_get( cfg, section1, "Init3");
	AxConf_set( cfg, section2, "Init3", val);

	val = AxConf_get( cfg, section1, "Init4");
	AxConf_set( cfg, section2, "Init4", val);

	val = AxConf_get( cfg, section1, "Dial Command");
	AxConf_set( cfg, section2, "Dial Command", val);
}
static void
format_read_func (gpointer	    data,
		  gint		    source,
		  GdkInputCondition condition)
{
	GtkWidget *dialog;
	GtkWidget *progress;
//	GtkAdjustment *adjustment;
	int size;
	char buf[LINE_MAX];
	gchar *str;
	gint len;
	gint percent;

	g_return_if_fail (GNOME_IS_DIALOG (data));
	dialog = GTK_WIDGET (data);
	memset(buf, '\0', sizeof(buf));
	while ((size = read(source,buf,sizeof(buf)-1)) != -1) {

		if(size == 0) {
			break;
		}

		buf[size] = '\000'; /* Make sure it's NULL terminated */

		str = buf;
		while (*str) {
			switch (str[0]) {
			case 'B': /* BC##BC */
				/* BC stands for Beginning Count */
				/* just as a sanity check...\n*/
				if (strlen (str) <5) {
					*str = '\000';
					break;
				}
				while(!isdigit(*str) && (*str != '\0')) {
					str++;
				}
				len = 0;
				while(isdigit(str[len]) && (str[len] != '\0')){
					len++;
				}
				str[len] = '\000'; /* Nuke the B  (-:  */
				progress = GTK_WIDGET (gtk_object_get_data (GTK_OBJECT (dialog), "progress"));
				GTK_PROGRESS (progress)->adjustment->upper = atoi (str);
				str += (len + 2); /*go past the second BC */
				break;

			case 'E': /* EC##EC */
				/* EC stands for End Count */
				/* just as a sanity check...\n*/
				if (strlen (str) <5) {
					*str = '\000';
					break;
				}
				str += 2;
				if (str[1] == 'E')
					len = 1;
				else
					len = 2;
				str[len] = '\000'; /* Nuke the E  (-:  */
				end_count = atoi (str);
				str += (len + 2); /*go past the second EC */
				break;

			case 'U': /* UPDATE */
				/* Sanity check */
				progress = GTK_WIDGET (gtk_object_get_data (GTK_OBJECT (dialog), "progress"));
				if ((strlen (str) < strlen ("UP###UP")) ||
				    (progress == NULL))  {
					*str = '\000';
					break;
				}
				str += strlen ("UP");
				str[3] = '\000';
				percent = atoi (str);
				gtk_progress_set_percentage (GTK_PROGRESS (progress),
							     CLAMP (percent/100.0, 0.0, 1.0));
				str += 5;
				break;

			case 'D': /* DONE */
				/* Sanity check */
				if ((strlen (str) >= strlen ("DONE")) &&
				    (strncmp (str, "DONE", strlen ("DONE")) == 0))  {
					gtk_widget_destroy (GTK_WIDGET (data));
				}
				*str = '\000';/* force an exit, even if we aren't through with the read. */
				break;
			default:
				break;
			}
		}
		memset(buf, '\0', sizeof(buf));
	}
	if (pid == 0) {
		/* Catch if our child dies */
		gtk_input_remove (format_input_handler);
		format_input_handler = 0;
	}
}

static const gchar *
strrstr (const gchar *haystack, const gchar *needle)
{
	const gchar *str;
	const gchar *ptr;

	ptr = NULL;
	str = haystack;

	while ((str = strstr (str, needle)) != NULL) {
		if (*str == '\000')
			break;
		ptr = str;
		str += 1;
	}
	return ptr;
}

static void
msg_read_func (gpointer	    data,
	       gint		    source,
	       GdkInputCondition condition)
{
	GtkWidget *dialog;
	GtkWidget *label;
//	GtkWidget *progress;
	int size;
	char buf[LINE_MAX];
	const gchar *str;

	g_return_if_fail (GNOME_IS_DIALOG (data));
	dialog = GTK_WIDGET (data);
	label = GTK_WIDGET (gtk_object_get_data (GTK_OBJECT (dialog), "label"));
	while ((size = read(source,buf,sizeof(buf)-1))>0) {
		buf[size] = '\000'; /* Make sure it's NULL terminated */

		str = strrstr (buf, "NEW_MSG");
		if ((str == NULL) || (strlen (str) < strlen ("NEW_MSG")))
			continue;
		gtk_label_set_text (GTK_LABEL (label), str + strlen ("NEW_MSG"));
	}
	if (pid == 0) {
		/* Catch if our child dies */
		gtk_input_remove (msg_input_handler);
		msg_input_handler = 0;
	}
}

void
msg_write_func (AxModemScanList *l, int fd)
{
	gchar *msg = NULL;
	GSList * list;
	AxModemScan * ptr;

	list = l->list;

	if (AxModemScanList_isdone(l))
	  return;

	for(; list;list=g_slist_next(list) ) {
		if (!AxModemScan_isdone((AxModemScan*)list->data)) break;
	}

	ptr = (AxModemScan*)list->data;
	switch ( AxModemScan_get_stage(ptr)) {
		case AxModemScan_Startup:
			msg = g_strdup_printf (_("%sLooking at %s"), "NEW_MSG",(const gchar *)AxModemScan_filename(ptr));
			break;
		case AxModemScan_AT:
		case AxModemScan_ATZ:
		case AxModemScan_ATS0:
		case AxModemScan_Carrier:
			msg = g_strdup_printf (_("%sChecking for modem and dial tone"), "NEW_MSG");
			break;
		case AxModemScan_DTR:
		case AxModemScan_FastDial:
		case AxModemScan_FCLASS:
		case AxModemScan_GetIdent:
			msg = g_strdup_printf (_("%sDetermining modem type"), "NEW_MSG");
			break;
		case AxModemScan_BaudStep:
		case AxModemScan_Reinit:
			msg = g_strdup_printf (_("%sDetermining modem speed"), "NEW_MSG");
			break;
		default:
			msg = g_strdup_printf (_("%sFinished looking at %s"), "NEW_MSG",(const gchar *)AxModemScan_filename(ptr));
	}

	if (msg) {
		write (fd, msg, strlen (msg));
		g_free (msg);
	}
}

static void signal_handler (int parm)
{
	pid = 0;
}

void
create_modem_dialog (GtkWidget *parent, gchar *section)
{
	GtkWidget *dialog;
	GtkWidget *hbox;
	GtkWidget *vbox;
	GtkWidget *logo;
	GtkWidget *label;
	GtkWidget *progress;
//	GtkRcStyle *rc_style;
	gint msg_pipe[2];
	gint format_pipe[2];
//	gint status;

	end_count = 0;

	dialog = gnome_dialog_new (_("Searching for modems"), GNOME_STOCK_BUTTON_CANCEL, NULL);
	gtk_window_set_default_size (GTK_WINDOW (dialog), 400, 300);
	logo = gnome_pixmap_new_from_file_at_size (ICON_PNG, 48, 48);

	gtk_widget_show (logo);
//	gtk_widget_set_usize (logo, 48, 48);

	hbox = gtk_hbox_new (FALSE, GNOME_PAD);
	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), hbox, TRUE, TRUE, 0);
	if (logo)
		gtk_box_pack_start (GTK_BOX (hbox), logo, FALSE, FALSE, GNOME_PAD_BIG);

	vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
	gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
	label = gtk_label_new (_("Searching for any modems..."));
	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
	label = gtk_label_new ("");
	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
	gtk_object_set_data (GTK_OBJECT (dialog), "label", label);

	progress = gtk_progress_bar_new ();
	gtk_box_pack_start (GTK_BOX (vbox), progress, FALSE, FALSE, 0);
	gtk_object_set_data (GTK_OBJECT (dialog), "progress", progress);
	gtk_widget_show_all (GNOME_DIALOG (dialog)->vbox);

	if ((pipe (msg_pipe) < 0) || (pipe (format_pipe) < 0))  {
		g_error ("unable to make a pipe\nexitting...\n");
		sgdial_exit (1);
	}

	msg_input_handler = gtk_input_add_full (msg_pipe[0], GDK_INPUT_READ, msg_read_func, NULL, dialog, NULL);
	format_input_handler = gtk_input_add_full (format_pipe[0], GDK_INPUT_READ, format_read_func, NULL, dialog, NULL);
	fcntl(msg_pipe[0], F_SETFL, O_NONBLOCK);
	fcntl(format_pipe[0], F_SETFL, O_NONBLOCK);
	signal (SIGCHLD, signal_handler);

	pid = fork ();
	if (pid < 0) {
		g_error ("unable to fork().\nPlease free up some resources and try again.\n");
		sgdial_exit (1);
	}
	if (pid == 0) {
		guint count;
		gfloat percent;
		gchar *msg;
		GSList *check;
		AxModemScan * ptr;

		AxModemScanList l;

		close (msg_pipe[0]);
		close (format_pipe[0]);

		AxModemScanList_init( &l );

		count = g_slist_length(l.list);
		msg = g_strdup_printf ("BC%dBC", count);
		write (format_pipe[1], msg, strlen (msg));
		g_free (msg);

//		check = l.list;

		while (!AxModemScanList_isdone(&l)) {
			AxModemScanList_execute(&l);
			percent = 1.0;

			msg_write_func(&l, msg_pipe[1]);
			for (check=l.list; check; check=g_slist_next(check)) {
				if (count == 0)
					continue;
				ptr = (AxModemScan*)check->data;
				if ( AxModemScan_get_stage(ptr) == 0)
					percent -= 1/(float)count;
				else
					percent -= (1/(float)count - (((float)AxModemScan_get_stage(ptr))/ AxModemScan_NUM_STAGES) * (1/(float) count));
			}
			CLAMP (percent, 0.0, 1.0);
			/* we guarentee 3 digits */
			msg = g_strdup_printf ("UP%.3dUP", (gint) (percent*100));
			write (format_pipe[1], msg, strlen (msg));
			g_free (msg);
		}
		msg = g_strdup_printf ("EC%dEC", g_slist_length(l.list));
		write (format_pipe[1], msg, strlen (msg));
		g_free (msg);

		/* let's store to disk */
		if (g_slist_length(l.list) > 0) {
			GSList * i = l.list;
			gint j = 0;
			char temp_section[18];
			AxModemScan *m;
			char * fn, *init;
			AxConf cfg;
			AxConf_init_valued(&cfg,get_config_path());

			for (; i;i=g_slist_next(i)) {
				/* We overwrite all other modems when we do this. */
				if (section)
					g_snprintf (temp_section, 17, "%s", section);
				else
					g_snprintf (temp_section, 17, "Modem%d", j);

				m = (AxModemScan*)i->data;
				fn = AxModemScan_filename(m);
				init = AxModemScan_initstr(m);

				AxConf_set(&cfg, temp_section, "Modem", fn);
				AxConf_setint(&cfg, temp_section, "Baud", AxModemScan_maxbaud(m));
				AxConf_set(&cfg, temp_section, "Init1", "ATZ");
				AxConf_set(&cfg, temp_section, "Init2", init);
				j++;
			}
			AxConf_clear( &cfg );
		}
		write (format_pipe[1], "DONE", strlen ("DONE"));
		close (msg_pipe[1]);
		close (format_pipe[1]);

		AxModemScanList_clear( & l );
		_exit (0);
	}

	/* parent */
	close (msg_pipe[1]);
	close (format_pipe[1]);
	gtk_widget_set_sensitive (parent, FALSE);
	gnome_dialog_set_parent (GNOME_DIALOG (dialog), GTK_WINDOW (parent));
	gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
	if (pid != 0) {
		/* I don't know if this is good. */
		/* it could leave us in a broken state. */
		kill (pid, 9);
		pid = 0;
		gtk_input_remove (msg_input_handler);
		gtk_input_remove (format_input_handler);
		msg_input_handler = 0;
		format_input_handler = 0;
		end_count = 0;
	}
	signal (SIGCHLD, SIG_DFL);
	close (msg_pipe[0]);
	close (format_pipe[0]);

	gtk_widget_set_sensitive (parent, TRUE);
}

GSList *
load_modems ( void )
{
	GSList *retval = NULL;

	AxConf cfg;
  GSList * list; 
  AxConfigSection * section;

	AxConf_init_valued(&cfg,get_config_path());

	for( list=cfg.sections; list; list=g_slist_next(list) ) {
		section = (AxConfigSection*)list->data;
		if (strncmp(section->name, "Modem", strlen ("Modem")) == 0) {
			Modem *modem = (Modem *) g_malloc ( sizeof(Modem) );
			modem->device= g_strdup(AxConfigSection_defget(section,"Modem", ""));
			modem->speed = g_strdup(AxConfigSection_defget(section,"Baud", "460800"));
			modem->section=g_strdup(section->name);
			retval = g_slist_prepend (retval, modem);
		}
	}
	return retval;
}

void
modem_destroy (Modem *modem)
{
	AxConf cfg;
//	gchar section[10];
//	gint i, j;

	AxConf_init_valued(&cfg,get_config_path());

	g_assert (modem->section != NULL);
	g_assert (strlen (modem->section) > strlen ("Modem"));

	AxConf_delete_section(&cfg,modem->section);
	g_free (modem->device);
	g_free (modem->speed);
	g_free (modem->section);
	g_free (modem);
}

void
modem_save (Modem *modem)
{
	g_print ("UNIMPLEMENTED:  Save modem here\n");
}


